package clsp.aligner;

import static clsp.util.Logger.logs;

import java.util.ArrayList;

import clsp.util.Filter;
import clsp.util.Filters;
import clsp.util.Lists;
import clsp.util.Logger;
import clsp.util.Option;
import clsp.util.SentencePair;
import clsp.util.SentencePairReader;
import clsp.util.SentencePairReader.PairDepot;

public class Data {
	// Input parameters
	@Option(gloss = "Directories or files containing training files.")
	public ArrayList<String> trainSources = Lists.newList("example/train");
	@Option(gloss = "Directory or file containing testing files.")
	public ArrayList<String> testSources = Lists.newList("example/test");
	@Option(name = "sentences", gloss = "Maximum number of the training sentences to use")
	public int maxTrainSentences = Integer.MAX_VALUE;
	@Option(gloss = "Skip this number of the first training sentences")
	public int offsetTrainingSentences = 0;
	@Option(gloss = "Maximum length (in words) of a training sentence")
	public static int maxTrainingLength = 200;
	@Option(gloss = "Maximum number of the test sentences to use")
	public int maxTestSentences = Integer.MAX_VALUE;
	@Option(gloss = "Skip this number of the first test sentences")
	public int offsetTestSentences = 0;

	@Option(gloss = "Foreign language file suffix")
	public static String foreignSuffix = "es";
	@Option(gloss = "English language file suffix")
	public static String englishSuffix = "en";
	@Option(gloss = "Convert all words to lowercase")
	public boolean lowercaseWords = false;
	@Option(gloss = "Don't load and store the training set upfront (slower, but less memory)")
	public boolean leaveTrainingOnDisk = false;
	
	PairDepot testPairs, trainingPairs;
	
	public void loadData() {
		// Read (or prepare to read) data
		Logger.track("Preparing Training Data");
		SentencePairReader spReader = new SentencePairReader(lowercaseWords);
		spReader.setEnglishExtension(englishSuffix);
		spReader.setForeignExtension(foreignSuffix);
		testPairs = spReader.pairDepotFromSources(testSources,
				offsetTestSentences, maxTestSentences, getTestFilter(), false);
		trainingPairs = spReader.pairDepotFromSources(trainSources,
				offsetTrainingSentences, maxTrainSentences, getTrainingFilter(),
				leaveTrainingOnDisk);
		String trainSize = "Unknown number of";
		if (!leaveTrainingOnDisk) trainSize = ((Integer) trainingPairs.size()).toString();
		Logger.end_track();
		logs("%s training sentences, %d test sentences", trainSize, testPairs.size());
	}

	private Filter<SentencePair> getTestFilter() {
		return Filters.andFilter(getTrainingFilter(), new Filter<SentencePair>() {
			public boolean accept(SentencePair t) {
				return t.getAlignment() != null;
			}
		});
	}

	private Filter<SentencePair> getTrainingFilter() {
		return Filters.acceptFilter(SentencePair.class);
	}
}
