package clsp.aligner;

import java.util.List;
import java.util.Map.Entry;

import clsp.util.Alignment;
import clsp.util.Fmt;
import clsp.util.ListUtils;
import clsp.util.Logger;
import clsp.util.NumUtils;
import clsp.util.SentencePair;
import clsp.util.String2DoubleMap;
import clsp.util.StringDoubleMap;
import clsp.util.SentencePairReader.PairDepot;

public class Model1 extends Model {

	public Model1(boolean reverse) {
		super(reverse);
	}

	String2DoubleMap parameters = new String2DoubleMap();
	String2DoubleMap expectations = new String2DoubleMap();
	double dataLogLikelihood = 0;

	@Override
	public void observe(SentencePair pair) {
		/* Solution */
		List<String> ens = reverse ? pair.getForeignWords() : pair.getEnglishWords();
		List<String> frs = !reverse ? pair.getForeignWords() : pair.getEnglishWords();
		for (int fpos = 0; fpos < frs.size(); fpos++) {
			String fr = frs.get(fpos);
			double[] posteriors = new double[ens.size()];
			for (int epos = 0; epos < ens.size(); epos++) {
				String en = ens.get(epos);
				posteriors[epos] = 1.0 / ens.size() * parameters.getSure(en, fr);
			}
			dataLogLikelihood += Math.log(ListUtils.sum(posteriors));
			NumUtils.normalize(posteriors);
			for (int epos = 0; epos < ens.size(); epos++) {
				String en = ens.get(epos);
				expectations.incr(en, fr, posteriors[epos]);
			}
		}

		// TODO Collect expected statistics about the sentence pair
	}

	@Override
	public Alignment score(SentencePair pair) {
		Alignment alignment = new Alignment(pair);

		/* Solution */
		List<String> ens = reverse ? pair.getForeignWords() : pair.getEnglishWords();
		List<String> frs = !reverse ? pair.getForeignWords() : pair.getEnglishWords();
		for (int fpos = 0; fpos < frs.size(); fpos++) {
			String fr = frs.get(fpos);
			double[] posteriors = new double[ens.size()];
			for (int epos = 0; epos < ens.size(); epos++) {
				String en = ens.get(epos);
				posteriors[epos] = 1.0 / ens.size() * parameters.getSure(en, fr);
			}
			NumUtils.normalize(posteriors);
			for (int epos = 0; epos < ens.size(); epos++) {
			  if (reverse) {
  				alignment.setStrength(fpos, epos, posteriors[epos]);
			  } else {
  				alignment.setStrength(epos, fpos, posteriors[epos]);
			  }
			}
		}

		// TODO Return the posterior distribution over alignments
		return alignment;
	}

	@Override
	public void completeIteration(int i) {
		/* Solution */
		Logger.logss("Iteration %d, Log-likelihood=%s", i + 1, Fmt.D(dataLogLikelihood));
		for (Entry<String, StringDoubleMap> entry : expectations) {
			entry.getValue().multAll(1.0 / entry.getValue().sum());
		}
		parameters = expectations;
		expectations = new String2DoubleMap();
		wordPairStats.allocateForSentencePairs(expectations, reverse);
		dataLogLikelihood = 0;
		
		// TODO Normalize the distributions for each word		
	}

	@Override
	protected void initModel(PairDepot trainingPairs) {
		// Initialize map to hold expected counts
		wordPairStats.allocateForSentencePairs(expectations, reverse);
		wordPairStats.allocateForSentencePairs(parameters, reverse);

		/* Solution */
		for (Entry<String, StringDoubleMap> entry : parameters) {
			String en = entry.getKey();
			StringDoubleMap map = entry.getValue();
			for (String fr : map.keySet()) {
				if (fr != null) parameters.put(en, fr, 1.0 / map.size());
			}
		}

		// TODO Initialize the model
	}
}
